@@ -29,9 +29,9 @@ Test::Lintian -- Check Lintian files for issues
  use Test::Lintian;
  use Test::More import => ['done_testing'];
  test_load_profiles('some/path');
- 
+
  done_testing;
- 
+
  # file 2
  use Test::Lintian;
  use Test::More import => ['done_testing'];
@@ -39,7 +39,7 @@ Test::Lintian -- Check Lintian files for issues
  test_check_desc('some/path/checks');
  test_load_checks('some/path/checks');
  test_tags_implemented('some/path/checks');
- 
+
  done_testing;
 
 =head1 DESCRIPTION
@@ -70,14 +70,16 @@ use parent 'Test::Builder::Module';
 use Cwd qw(realpath);
 use File::Basename qw(basename);
 use File::Find ();
+use List::MoreUtils qw{any};
 use Path::Tiny;
 
-use Lintian::Check qw(check_spelling);
+use Lintian::Spelling qw(check_spelling);
 use Lintian::Data;
+use Lintian::Deb822Parser qw(read_dpkg_control);
 use Lintian::Profile;
 use Lintian::Tag::Info;
 use Lintian::Tags;
-use Lintian::Util qw(read_dpkg_control is_string_utf8_encoded);
+use Lintian::Util qw(is_string_utf8_encoded);
 
 # We want data files loaded early to avoid problems with missing data
 # files ending up in releases (like in 2.5.17 and 2.5.18).
@@ -466,16 +468,18 @@ sub test_load_checks {
         $ppkg =~ s,[-.],_,go;
         $ppkg =~ s,/,::,go;
         $ppkg = "Lintian::$ppkg";
-        eval {
-            # minimal "no strict refs" scope.
-            no strict 'refs';
-            $rs_ref = 'PRESENT'
-              if defined &{$ppkg . '::run'};
-        };
-        $err = $@//'';
-        if (!$builder->is_eq($rs_ref, 'PRESENT', "Check $cname has runsub")) {
+
+        if ($ppkg->can('run') && !$ppkg->DOES('Lintian::Check')) {
+            $builder->diag(
+                "Warning: check $ppkg uses old entry point ::run\n");
+        }
+
+        # setup and breakdown should only be used together with files
+        my $has_entrypoint = any { $ppkg->can($_) }
+        ('source', 'binary', 'udeb', 'changes', 'always', 'files');
+
+        if (!$builder->ok($has_entrypoint, "Check $cname has entry point")) {
             $builder->diag("Expected package name is $ppkg\n");
-            $builder->diag("Error: $err\n") if $err;
         }
     }
     return;
@@ -649,10 +653,10 @@ sub load_profile_for_test {
      # do just fine...
     $profname ||= 'debian/main';
 
-    unless (@inc) {
-        push @inc, $ENV{'LINTIAN_TEST_ROOT'} if $ENV{'LINTIAN_TEST_ROOT'};
-        push @inc, '/usr/share/lintian' if -d '/usr/share/lintian';
-    }
+    my $location
+      = $ENV{'LINTIAN_ROOT'} ? $ENV{'LINTIAN_ROOT'} : '/usr/share/lintian';
+    push(@inc, $location)
+      unless @inc;
 
     $PROFILE = Lintian::Profile->new($profname, \@inc);
     Lintian::Data->set_vendor($PROFILE);
@@ -807,3 +811,9 @@ files/symlinks beneath it.
 =cut
 
 1;
+
+# Local Variables:
+# indent-tabs-mode: nil
+# cperl-indent-level: 4
+# End:
+# vim: syntax=perl sw=4 sts=4 sr et
